/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.petals;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import org.ow2.weblab.core.services.AccessDeniedException;
import org.ow2.weblab.core.services.ContentNotAvailableException;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.Trainable;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.UnsupportedRequestException;
import org.ow2.weblab.core.services.trainable.AddTrainResourceArgs;
import org.ow2.weblab.core.services.trainable.AddTrainResourceReturn;
import org.ow2.weblab.core.services.trainable.ResetTrainedModelArgs;
import org.ow2.weblab.core.services.trainable.ResetTrainedModelReturn;
import org.ow2.weblab.core.services.trainable.TrainArgs;
import org.ow2.weblab.core.services.trainable.TrainReturn;

public class JbiTrainable implements Trainable {
	private static final String JAXB_ERROR = "Jaxb error, unable to instanciate the response writer.";
	private static final String ERROR_DURING_CALL = "Error during call";
	private static final String TRAINABLE_NS = "http://weblab.ow2.org/core/1.2/services/trainable";
	protected AbstractMethod<AddTrainResourceArgs, AddTrainResourceReturn> innerJbiAddTrainResource;
	protected AbstractMethod<TrainArgs, TrainReturn> innerJbiTrain;
	protected AbstractMethod<ResetTrainedModelArgs, ResetTrainedModelReturn> innerJbiResetTrainedModel;
	protected Logger logger;

	public JbiTrainable(final QName serviceName, final ComponentContext ctx, final Logger logger) throws JAXBException {
		this.logger = logger;
		innerJbiAddTrainResource = new AbstractMethod<AddTrainResourceArgs, AddTrainResourceReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(TRAINABLE_NS, "addTrainResourceReturn");
			}

			@Override
			protected Class<AddTrainResourceReturn> getParameterRetClass() {
				return AddTrainResourceReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(TRAINABLE_NS, "addTrainResourceArgs");
			}

			@Override
			protected Class<AddTrainResourceArgs> getParameterArgsClass() {
				return AddTrainResourceArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("addTrainResource");
			}

		};

		innerJbiTrain = new AbstractMethod<TrainArgs, TrainReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(TRAINABLE_NS, "trainReturn");
			}

			@Override
			protected Class<TrainReturn> getParameterRetClass() {
				return TrainReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(TRAINABLE_NS, "trainArgs");
			}

			@Override
			protected Class<TrainArgs> getParameterArgsClass() {
				return TrainArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("train");
			}
		};

		innerJbiResetTrainedModel = new AbstractMethod<ResetTrainedModelArgs, ResetTrainedModelReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(TRAINABLE_NS, "resetTrainedModelReturn");
			}

			@Override
			protected Class<ResetTrainedModelReturn> getParameterRetClass() {
				return ResetTrainedModelReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(TRAINABLE_NS, "resetTrainedModelArgs");
			}

			@Override
			protected Class<ResetTrainedModelArgs> getParameterArgsClass() {
				return ResetTrainedModelArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("resetTrainedModel");
			}
		};
	}

	@Override
	public AddTrainResourceReturn addTrainResource(final AddTrainResourceArgs args) throws AccessDeniedException, ContentNotAvailableException,
			InsufficientResourcesException, InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiAddTrainResource.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, ERROR_DURING_CALL, e);

			throw new UnexpectedException(ERROR_DURING_CALL, e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, JAXB_ERROR, e);

			throw new UnexpectedException(JAXB_ERROR, e.getMessage(), e);
		}

	}

	@Override
	public TrainReturn train(final TrainArgs args) throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException,
			InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiTrain.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, ERROR_DURING_CALL, e);

			throw new UnexpectedException(ERROR_DURING_CALL, e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, JAXB_ERROR, e);

			throw new UnexpectedException(JAXB_ERROR, e.getMessage(), e);
		}

	}

	@Override
	public ResetTrainedModelReturn resetTrainedModel(final ResetTrainedModelArgs args) throws AccessDeniedException, ContentNotAvailableException,
			InsufficientResourcesException, InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiResetTrainedModel.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, ERROR_DURING_CALL, e);

			throw new UnexpectedException(ERROR_DURING_CALL, e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, JAXB_ERROR, e);

			throw new UnexpectedException(JAXB_ERROR, e.getMessage(), e);
		}

	}

}
